﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace BasicApplicationTests.TransportService
{
    internal class DataProvider
    {
        public const int MAX_FRAME_SIZE = 30;

        // Datagram size = 377 (1 0111 1001)
        public static ReadOnlyCollection<byte> Datagram = Array.AsReadOnly(new byte[] { 0xE4, 0x3D, 0x92, 0x48, 0x59, 0x8F, 0xFD, 0x2F, 0x8C, 0xA8, 0xD5, 0x0C, 0x87, 0xD4, 0x83, 0x99, 0x37, 0xD6, 0xD6, 0x46, 0x4B, 0xB2, 0xB9, 0x50, 0x86,
                0x73, 0x9B, 0xD6, 0xAB, 0x2A, 0xFE, 0x8F, 0xFB, 0xF7, 0xC7, 0xFE, 0x69, 0xC3, 0x5A, 0xE3, 0xA8, 0x32, 0x07, 0x7C, 0x93, 0xCD, 0x24, 0xB9, 0x83, 0x75,
                0x34, 0x5F, 0xBC, 0x5E, 0xD8, 0xF6, 0xD9, 0xA5, 0x7D, 0x65, 0x59, 0xCA, 0x0F, 0x9E, 0xAE, 0xA9, 0xB9, 0xAD, 0xDD, 0x3D, 0x4F, 0x51, 0x06, 0x9D, 0xAD, 0x43, 0x65, 0x13, 0x86, 0x85, 0x87, 0x6F, 0xEF,
                0x2B, 0xB9, 0x9E, 0x8D, 0xF5, 0x80, 0xC7, 0x8C, 0xA3, 0x58, 0x03, 0x33, 0xA8, 0x3C, 0x7E, 0x73, 0xE2, 0xC2, 0xF3, 0xAC, 0x37, 0x8D, 0x14, 0xD6, 0xDE, 0x71, 0x64, 0x42, 0xE3, 0xB9, 0x03, 0xDC, 0xF4,
                0x97, 0x38, 0xBC, 0xC8, 0xA5, 0x2C, 0x9F, 0x1A, 0x7B, 0x68, 0x29, 0xFF, 0x30, 0x81, 0x96, 0xBD, 0x61, 0xAB, 0xAC, 0xFC, 0x2C, 0x7B, 0x90, 0x08, 0x3E, 0xA8, 0x9D, 0x54, 0xB3, 0xEA, 0x70, 0x86, 0x0A,
                0xBE, 0x05, 0x2A, 0x8B, 0xB0, 0x4E, 0x53, 0x7E, 0x2F, 0xE0, 0xDB, 0xD7, 0x36, 0x7C, 0x7C, 0xB3, 0xFC, 0x1B, 0x52, 0x6D, 0xF5, 0x18, 0x95, 0x7F, 0x92, 0xEA, 0x1A, 0x54, 0x31, 0x81, 0xE7, 0x6F, 0x6A,
                0x14, 0xD7, 0x84, 0x95, 0xD8, 0xD5, 0x5F, 0x09, 0x38, 0x92, 0xBF, 0xCA, 0x51, 0xF5, 0x19, 0xA8, 0x1C, 0xE4, 0xB5, 0x93, 0x5D, 0x94, 0x64, 0xC7, 0x92, 0xA1, 0xCD, 0x05, 0xCC, 0xBD, 0x0B, 0x39, 0xD9,
                0x19, 0xFB, 0x3D, 0xE9, 0xD7, 0x8D, 0x31, 0xCA, 0x3D, 0xA2, 0xF8, 0xA0, 0x48, 0xA3, 0x4D, 0xFD, 0x30, 0xAC, 0x9C, 0x4E, 0xEB, 0x80, 0x49, 0xEC, 0xB1, 0xCB, 0x5A, 0x2B, 0x45, 0x75, 0x16, 0x7D, 0xB3,
                0x0B, 0xE6, 0xE8, 0x1B, 0xF3, 0xF2, 0x32, 0xFA, 0x7C, 0x4A, 0x49, 0xC7, 0xE7, 0x28, 0xB0, 0x0D, 0x91, 0xE6, 0x9B, 0xB2, 0xE6, 0x9C, 0xB9, 0x7B, 0x0C, 0xB0, 0xC0, 0xE2, 0xCE, 0xD9, 0xAA, 0x2C, 0x64,
                0x96, 0x2D, 0xFF, 0x02, 0x62, 0x99, 0x77, 0xEC, 0x29, 0x4F, 0xBE, 0x19, 0xE9, 0x04, 0xE7, 0xBA, 0xF1, 0x30, 0x34, 0xAB, 0x78, 0xDB, 0x8A, 0xA3, 0x11, 0x6B, 0x35, 0x2B, 0xF0, 0xF0, 0xA1, 0xBC, 0x6C,
                0x7B, 0xD6, 0xAA, 0x0A, 0xCC, 0xE3, 0x75, 0x2B, 0xD2, 0x0C, 0xCE, 0xEF, 0x79, 0x30, 0x45, 0x41, 0x0C, 0xE6, 0x2A, 0xD3, 0x74, 0x61, 0x05, 0x62, 0x19, 0xEA, 0x7C, 0x53, 0xA5, 0x0E, 0xBA, 0x79, 0x8F,
                0xA7, 0xF4, 0x59, 0x39, 0xBA, 0x9E, 0x44, 0x6C, 0x36, 0xED, 0x9D, 0xEF, 0x2A, 0xE4, 0x3B, 0x06, 0x95, 0xCF, 0x63, 0x5D, 0xA5, 0x4F, 0x77, 0x4F, 0x37, 0x41, 0x57, 0x91, 0x31, 0xF5 });

        // Session id = 2.
        public static byte SessionId { get { return 2; } }

        private static Dictionary<ushort, byte[]> _segmentsByOffsets;
        public static Dictionary<ushort, byte[]> SegmentsByOffsets { get { return _segmentsByOffsets; } }

        public static ReadOnlyCollection<byte[]> SegmentCmds { get { return Array.AsReadOnly<byte[]>(_segmentsByOffsets.Values.ToArray()); } }

        static DataProvider()
        {
            _segmentsByOffsets = new Dictionary<ushort, byte[]>();
            _segmentsByOffsets.Add(0,
                // Segment 0
                // ext = 1
                // offset = 0
                // header extension length = 8
                // header extension { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 }
                // payload size = 25
                // first.
                // 0xE4, 0x3D, 0x92, 0x48, 0x59, 0x8F, 0xFD, 0x2F, 0x8C, 0xA8, 0xD5, 0xC, 0x87, 0xD4, 0x83, 0x99, 0x37, 0xD6, 0xD6, 0x46, 0x4B, 0xB2, 0xB9, 0x50, 0x86
                new byte[] { 0x55, 0xC1, 0x79, 0x28, 0x08, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0xE4, 0x3D, 0x92, 0x48, 0x59, 0x8F, 0xFD, 0x2F, 0x8C, 0xA8, 0xD5, 0xC, 0x87, 0xD4, 0x83, 0x99, 0x37, 0xD6, 0xD6, 0x46, 0x4B, 0xB2, 0xB9, 0x50, 0x86, 0x4A, 0xB9 }
                );
            _segmentsByOffsets.Add(25,
                // Segment 1
                // ext = 1
                // offset = 25
                // header extension length = 7
                // header extension { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 }
                // payload size = 25
                // 0x73, 0x9B, 0xD6, 0xAB, 0x2A, 0xFE, 0x8F, 0xFB, 0xF7, 0xC7, 0xFE, 0x69, 0xC3, 0x5A, 0xE3, 0xA8, 0x32, 0x7, 0x7C, 0x93, 0xCD, 0x24, 0xB9, 0x83, 0x75
                new byte[] { 0x55, 0xC9, 0x79, 0x28, 0x19, 0x07, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x73, 0x9B, 0xD6, 0xAB, 0x2A, 0xFE, 0x8F, 0xFB, 0xF7, 0xC7, 0xFE, 0x69, 0xC3, 0x5A, 0xE3, 0xA8, 0x32, 0x7, 0x7C, 0x93, 0xCD, 0x24, 0xB9, 0x83, 0x75, 0xF9, 0x6C }
                );
            _segmentsByOffsets.Add(50,
                // Segment 2
                // offset = 50
                // payload size = 33
                // 0x34, 0x5F, 0xBC, 0x5E, 0xD8, 0xF6, 0xD9, 0xA5, 0x7D, 0x65, 0x59, 0xCA, 0xF, 0x9E, 0xAE, 0xA9, 0xB9, 0xAD, 0xDD, 0x3D, 0x4F, 0x51, 0x6, 0x9D, 0xAD, 0x43, 0x65, 0x13, 0x86, 0x85, 0x87, 0x6F, 0xEF
                new byte[] { 0x55, 0xC9, 0x79, 0x20, 0x32, 0x34, 0x5F, 0xBC, 0x5E, 0xD8, 0xF6, 0xD9, 0xA5, 0x7D, 0x65, 0x59, 0xCA, 0xF, 0x9E, 0xAE, 0xA9, 0xB9, 0xAD, 0xDD, 0x3D, 0x4F, 0x51, 0x6, 0x9D, 0xAD, 0x43, 0x65, 0x13, 0x86, 0x85, 0x87, 0x6F, 0xEF, 0x9C, 0x0E }
                );
            _segmentsByOffsets.Add(83,
                // Segment 3
                // offset = 83
                // payload size = 33
                // 0x2B, 0xB9, 0x9E, 0x8D, 0xF5, 0x80, 0xC7, 0x8C, 0xA3, 0x58, 0x3, 0x33, 0xA8, 0x3C, 0x7E, 0x73, 0xE2, 0xC2, 0xF3, 0xAC, 0x37, 0x8D, 0x14, 0xD6, 0xDE, 0x71, 0x64, 0x42, 0xE3, 0xB9, 0x3, 0xDC, 0xF4
                new byte[] { 0x55, 0xC9, 0x79, 0x20, 0x53, 0x2B, 0xB9, 0x9E, 0x8D, 0xF5, 0x80, 0xC7, 0x8C, 0xA3, 0x58, 0x3, 0x33, 0xA8, 0x3C, 0x7E, 0x73, 0xE2, 0xC2, 0xF3, 0xAC, 0x37, 0x8D, 0x14, 0xD6, 0xDE, 0x71, 0x64, 0x42, 0xE3, 0xB9, 0x3, 0xDC, 0xF4, 0x9F, 0x56 }
                );
            _segmentsByOffsets.Add(116,
                // Segment 4
                // offset = 116
                // payload size = 33
                // 0x97, 0x38, 0xBC, 0xC8, 0xA5, 0x2C, 0x9F, 0x1A, 0x7B, 0x68, 0x29, 0xFF, 0x30, 0x81, 0x96, 0xBD, 0x61, 0xAB, 0xAC, 0xFC, 0x2C, 0x7B, 0x90, 0x8, 0x3E, 0xA8, 0x9D, 0x54, 0xB3, 0xEA, 0x70, 0x86, 0x0A
                new byte[] { 0x55, 0xC9, 0x79, 0x20, 0x74, 0x97, 0x38, 0xBC, 0xC8, 0xA5, 0x2C, 0x9F, 0x1A, 0x7B, 0x68, 0x29, 0xFF, 0x30, 0x81, 0x96, 0xBD, 0x61, 0xAB, 0xAC, 0xFC, 0x2C, 0x7B, 0x90, 0x8, 0x3E, 0xA8, 0x9D, 0x54, 0xB3, 0xEA, 0x70, 0x86, 0x0A, 0x99, 0x7B }
                );
            _segmentsByOffsets.Add(149,
                // Segment 5
                // offset = 149
                // payload size = 33
                // 0xBE, 0x5, 0x2A, 0x8B, 0xB0, 0x4E, 0x53, 0x7E, 0x2F, 0xE0, 0xDB, 0xD7, 0x36, 0x7C, 0x7C, 0xB3, 0xFC, 0x1B, 0x52, 0x6D, 0xF5, 0x18, 0x95, 0x7F, 0x92, 0xEA, 0x1A, 0x54, 0x31, 0x81, 0xE7, 0x6F, 0x6A
                new byte[] { 0x55, 0xC9, 0x79, 0x20, 0x95, 0xBE, 0x5, 0x2A, 0x8B, 0xB0, 0x4E, 0x53, 0x7E, 0x2F, 0xE0, 0xDB, 0xD7, 0x36, 0x7C, 0x7C, 0xB3, 0xFC, 0x1B, 0x52, 0x6D, 0xF5, 0x18, 0x95, 0x7F, 0x92, 0xEA, 0x1A, 0x54, 0x31, 0x81, 0xE7, 0x6F, 0x6A, 0xC3, 0x3B }
                );
            _segmentsByOffsets.Add(182,
                // Segment 6
                // offset = 182
                // payload size = 33
                // 0x14, 0xD7, 0x84, 0x95, 0xD8, 0xD5, 0x5F, 0x9, 0x38, 0x92, 0xBF, 0xCA, 0x51, 0xF5, 0x19, 0xA8, 0x1C, 0xE4, 0xB5, 0x93, 0x5D, 0x94, 0x64, 0xC7, 0x92, 0xA1, 0xCD, 0x5, 0xCC, 0xBD, 0xB, 0x39, 0xD9
                new byte[] { 0x55, 0xC9, 0x79, 0x20, 0xB6, 0x14, 0xD7, 0x84, 0x95, 0xD8, 0xD5, 0x5F, 0x9, 0x38, 0x92, 0xBF, 0xCA, 0x51, 0xF5, 0x19, 0xA8, 0x1C, 0xE4, 0xB5, 0x93, 0x5D, 0x94, 0x64, 0xC7, 0x92, 0xA1, 0xCD, 0x5, 0xCC, 0xBD, 0xB, 0x39, 0xD9, 0x41, 0x29 }
                );
            _segmentsByOffsets.Add(215,
                // Segment 7
                // offset = 215
                // payload size = 33
                // 0x19, 0xFB, 0x3D, 0xE9, 0xD7, 0x8D, 0x31, 0xCA, 0x3D, 0xA2, 0xF8, 0xA0, 0x48, 0xA3, 0x4D, 0xFD, 0x30, 0xAC, 0x9C, 0x4E, 0xEB, 0x80, 0x49, 0xEC, 0xB1, 0xCB, 0x5A, 0x2B, 0x45, 0x75, 0x16, 0x7D, 0xB3
                new byte[] { 0x55, 0xC9, 0x79, 0x20, 0xD7, 0x19, 0xFB, 0x3D, 0xE9, 0xD7, 0x8D, 0x31, 0xCA, 0x3D, 0xA2, 0xF8, 0xA0, 0x48, 0xA3, 0x4D, 0xFD, 0x30, 0xAC, 0x9C, 0x4E, 0xEB, 0x80, 0x49, 0xEC, 0xB1, 0xCB, 0x5A, 0x2B, 0x45, 0x75, 0x16, 0x7D, 0xB3, 0x01, 0xD4 }
                );
            _segmentsByOffsets.Add(248,
                // Segment 8
                // offset = 248
                // payload size = 33
                // 0xB, 0xE6, 0xE8, 0x1B, 0xF3, 0xF2, 0x32, 0xFA, 0x7C, 0x4A, 0x49, 0xC7, 0xE7, 0x28, 0xB0, 0xD, 0x91, 0xE6, 0x9B, 0xB2, 0xE6, 0x9C, 0xB9, 0x7B, 0xC, 0xB0, 0xC0, 0xE2, 0xCE, 0xD9, 0xAA, 0x2C, 0x64
                new byte[] { 0x55, 0xC9, 0x79, 0x20, 0xF8, 0xB, 0xE6, 0xE8, 0x1B, 0xF3, 0xF2, 0x32, 0xFA, 0x7C, 0x4A, 0x49, 0xC7, 0xE7, 0x28, 0xB0, 0xD, 0x91, 0xE6, 0x9B, 0xB2, 0xE6, 0x9C, 0xB9, 0x7B, 0xC, 0xB0, 0xC0, 0xE2, 0xCE, 0xD9, 0xAA, 0x2C, 0x64, 0x05, 0x5E }
                );
            _segmentsByOffsets.Add(281,
                // Segment 9
                // offset = 281
                // payload size = 33
                // 0x96, 0x2D, 0xFF, 0x2, 0x62, 0x99, 0x77, 0xEC, 0x29, 0x4F, 0xBE, 0x19, 0xE9, 0x4, 0xE7, 0xBA, 0xF1, 0x30, 0x34, 0xAB, 0x78, 0xDB, 0x8A, 0xA3, 0x11, 0x6B, 0x35, 0x2B, 0xF0, 0xF0, 0xA1, 0xBC, 0x6C
                new byte[] { 0x55, 0xC9, 0x79, 0x21, 0x19, 0x96, 0x2D, 0xFF, 0x2, 0x62, 0x99, 0x77, 0xEC, 0x29, 0x4F, 0xBE, 0x19, 0xE9, 0x4, 0xE7, 0xBA, 0xF1, 0x30, 0x34, 0xAB, 0x78, 0xDB, 0x8A, 0xA3, 0x11, 0x6B, 0x35, 0x2B, 0xF0, 0xF0, 0xA1, 0xBC, 0x6C, 0x1B, 0xCA }
                );
            _segmentsByOffsets.Add(314,
                // Segment 10
                // offset = 314
                // payload size = 33
                // 0x7B, 0xD6, 0xAA, 0xA, 0xCC, 0xE3, 0x75, 0x2B, 0xD2, 0xC, 0xCE, 0xEF, 0x79, 0x30, 0x45, 0x41, 0xC, 0xE6, 0x2A, 0xD3, 0x74, 0x61, 0x5, 0x62, 0x19, 0xEA, 0x7C, 0x53, 0xA5, 0xE, 0xBA, 0x79, 0x8F
                new byte[] { 0x55, 0xC9, 0x79, 0x21, 0x3A, 0x7B, 0xD6, 0xAA, 0xA, 0xCC, 0xE3, 0x75, 0x2B, 0xD2, 0xC, 0xCE, 0xEF, 0x79, 0x30, 0x45, 0x41, 0xC, 0xE6, 0x2A, 0xD3, 0x74, 0x61, 0x5, 0x62, 0x19, 0xEA, 0x7C, 0x53, 0xA5, 0xE, 0xBA, 0x79, 0x8F, 0x34, 0xB3 }
                );
            _segmentsByOffsets.Add(347,
                // Segment 11
                // offset = 347
                // payload size = 30
                // 0xA7, 0xF4, 0x59, 0x39, 0xBA, 0x9E, 0x44, 0x6C, 0x36, 0xED, 0x9D, 0xEF, 0x2A, 0xE4, 0x3B, 0x6, 0x95, 0xCF, 0x63, 0x5D, 0xA5, 0x4F, 0x77, 0x4F, 0x37, 0x41, 0x57, 0x91, 0x31, 0xF5
                new byte[] { 0x55, 0xC9, 0x79, 0x21, 0x5B, 0xA7, 0xF4, 0x59, 0x39, 0xBA, 0x9E, 0x44, 0x6C, 0x36, 0xED, 0x9D, 0xEF, 0x2A, 0xE4, 0x3B, 0x6, 0x95, 0xCF, 0x63, 0x5D, 0xA5, 0x4F, 0x77, 0x4F, 0x37, 0x41, 0x57, 0x91, 0x31, 0xF5, 0x4E, 0xAD }
                );
        }
    }
}
