﻿using ZWave.Layers;
using ZWave.Layers.Transport;
using ZWave.Layers.Application;
using ZWave.ZnifferApplication.Devices;
using ZWave.Enums;
using System;

namespace ZWave.ZnifferApplication
{
    public class ZnifferApplicationLayer : ApplicationLayer
    {
        public ITransportLayer UdpTransportLayer { get; set; }

        public ZnifferApplicationLayer(ISessionLayer sessionLayer, IFrameLayer frameLayer, ITransportLayer transportLayer)
            : base(ApiTypes.Zniffer, sessionLayer, frameLayer, transportLayer)
        {
        }

        public Zniffer CreateZniffer()
        {
            var sessionId = NextSessionId();
            Zniffer ret = new Zniffer(sessionId, SessionLayer.CreateClient(), FrameLayer.CreateClient(), TransportLayer.CreateClient(sessionId));
            return ret;
        }

        public Zniffer CreateZniffer(IApplicationClient client)
        {
            Zniffer ret = new Zniffer(client.SessionId, client.SessionClient, client.FrameClient, client.TransportClient);
            return ret;
        }

        public Zniffer CreateUdpListeningZniffer()
        {
            if (UdpTransportLayer != null)
            {
                UdpTransportLayer.Listener = new UdpClientTransportListener();
                var sessionId = NextSessionId();
                Zniffer ret = new Zniffer(sessionId, SessionLayer.CreateClient(), FrameLayer.CreateClient(), UdpTransportLayer.CreateClient(sessionId));
                return ret;
            }
            return null;
        }
    }
}
