﻿using ZWave.BasicApplication.Enums;
using ZWaveController.ViewModels;

namespace ZWaveController.Commands
{
    public class AssociationCreateCommand : AssociationCommandBase
    {
        public AssociationCreateCommand(AssociationModel associationViewModel)
            : base(associationViewModel)
        {
            IsTargetSensitive = true;
            Text = "Create Association";
            SetCanExecute(param => MainVM.Controller != null &&
                AssociationModel.SelectedNodeIds.Count > 0 && AssociationModel.SelectedGroup != null
                );
        }

        public override CommandTypes CommandType
        {
            get { return CommandTypes.CmdZWaveSendData; }
        }

        protected override void ExecuteInner(object parameter)
        {
            // 2. Log 
            Log("Create Association started.");

            // 3. ViewModel method
            var result = CommandExecutionResult.Failed;
            var node = AssociationModel.SelectedGroup.ParentDevice.Device;
            byte groupId = AssociationModel.SelectedGroup.Id;
            result = MainVM.ControllerModel.AssociationCreate(node, groupId, AssociationModel.SelectedNodeIds);
            MainVM.ControllerModel.AssociationGet(node, groupId);
            if (AssociationModel.IsAssignReturnRoutes && AssociationModel.SelectedGroup.Nodes != null)
            {
                MainVM.ControllerModel.DeleteReturnRoute(node.Id);
                MainVM.ControllerModel.AssignReturnRoute(node.Id, AssociationModel.SelectedGroup.GetNodeIds(), out _token);
            }

            // 4. ViewModel result
            MainVM.LastCommandExecutionResult = result;
        }
    }
}
