﻿using System;
using System.Linq;
using Utils.UI.MVVM;
using ZWave;
using ZWave.BasicApplication.Devices;
using ZWave.BasicApplication.Enums;
using ZWave.Devices;
using ZWaveController.Models;
using ZWaveController.ViewModels;

namespace ZWaveController.Commands
{
    public class CommandBasicBase : CommandBaseEx
    {
        public CommandBasicBase(MainViewModel mainVM, VMBase model)
            : base(mainVM, model)
        {
        }

        protected ActionToken _token = null;

        public Device Controller
        {
            get { return (Device)MainVM.Controller; }
        }

        public virtual CommandTypes CommandType { get { return CommandTypes.None; } }

        public virtual void UpdateTargetDevice()
        {
            if (MainVM.Controller != null && !_isTargetDeviceFrozen)
            {
                if (MainVM.SelectedNode != null)
                {
                    TargetDevice = MainVM.SelectedNode.Item;
                    IsSleepingTarget = !MainVM.Controller.Network.IsDeviceListening(TargetDevice);
                }
            }
        }

        bool _isTargetDeviceFrozen = false;
        public void FreezeTargetDevice()
        {
            _isTargetDeviceFrozen = true;
        }

        public bool IsSleepingTarget { get; set; }
        public NodeTag TargetDevice { get; set; }
        public bool IsTargetSensitive { get; set; }
        public bool IsCancelAtController { get; set; }

        protected override void CancelInner(object parameter)
        {
            IsCancelling = true;
            if (!IsCancelAtController)
            {
                if (TargetDevice.Id > 0)
                    MainVM.ControllerModel.Cancel(TargetDevice.Id, _token);
            }
            else
                MainVM.ControllerModel.Cancel(0, _token);
        }

        public override bool IsSupportedAndReady()
        {
            bool ret = true;
            UpdateTargetDevice();

            if (MainVM.ControllerModel is BasicControllerModel)
            {
                if (CommandType != CommandTypes.None && MainVM.Controller != null && Controller.SupportedSerialApiCommands != null)
                {
                    ret = Controller.SupportedSerialApiCommands.Contains((byte)CommandType);
                    if (!ret)
                    {
                        LogError("{0} is not implemented in the Z-Wave Device", CommandType);
                    }
                }
                if (ret && IsTargetSensitive && TargetDevice != null)
                {
                    if (TargetDevice.Id > 0 && IsSleepingTarget)
                    {
                        ret = false;
                        IsSleepingTarget = false;
                        LogOk(String.Format("Command: {0} is queued for Node: {1}", this.Text, TargetDevice));
                        (MainVM.ControllerModel as BasicControllerModel).EnqueueCommand(TargetDevice, (CommandBase)this.Clone());
                    }
                }
            }
            return ret;
        }
    }
}
